// ------------------------------------------------------------
// File:        gov_timer.js
//
// Copyright:   (c) 2002 Darren John Dwyer, Sydney, Australia,
//              All Rights Reserved.
//
// ------------------------------------------------------------
// Timer interface
//
// This file provides a simple to use timer class.
//
// ------------------------------------------------------------
// Example:
//        var timer1 = new d_Timer( 'timer1' );
//
//        timer1.Interval = 50;
//        timer1.TimeOut = 500;
//
//        timer1.OnStart = AnimateFirstFrame();
//        timer1.OnTick = AnimateOtherFrames();
//        timer1.OnStop = AnimateLastFrame();
//
//        timer1.Start();
//
// ------------------------------------------------------------

// ------------------------------------------------------------
// Define Global Timer Variables
// ------------------------------------------------------------

var     d_Timer_Counter = 0;
var    d_Timer_Debug = true;

// ------------------------------------------------------------
// Object Definition
// ------------------------------------------------------------

function d_Timer( timer_name )
{

// ----------------------------------------------------
// Properties
// ----------------------------------------------------

    // ----------------------------------------------------
    // Property:     Name
    // Description: Must be passed into the constructor
    //        during class creation. Must be the
    //        same name as the variable used to
    //        hold the instance of the class.
    //
    // Usage:    var name = new d_Timer( 'name' );
    //
    // ----------------------------------------------------

    this.Name        = timer_name;

    // ----------------------------------------------------
    // Property:     Type
    // Description:    Always returns "d_Timer"
    //
    // Usage:    alert("timer1.Type=="+timer1.Type);
    //
    // ----------------------------------------------------

    this.Type        = "d_Timer";

    // ----------------------------------------------------
    // Property:     Id 
    // Description: A unique identifier automatically 
    //        assigned to each d_Timer() class.
    //
    // Usage:    if (timer1.Id==0)
    //            alert("timer1 is first timer");
    //
    // ----------------------------------------------------

    this.Id            = d_Timer_Counter++;

    // ----------------------------------------------------
    // Property:    Active
    // Description:    if true, timer is currently running.
    //
    // Usage:    if (timer1.Active)
    //            alert("timer1 is running");
    //
    // ----------------------------------------------------

    this.Active        = false;

    // ----------------------------------------------------
    // Property:    Interval
    // Description:    Used to modify the rate at which the
    //        timer calls the .OnTick() event.
    //        Should always be set between 1..N,
    //        where N is a big number.
    //
    // Usage:    timer1.Interval = 1;    // 1 millisec
    //        timer1.Interval = 1000;    // 1 second
    //
    // ----------------------------------------------------

    this.Interval        = 1;

    // ----------------------------------------------------
    // Property:    Increment
    // Description:    Used to modify the value added to 
    //        .TickCount during each .Tick()
    //
    // Usage:    timer1.Increment = 1;    // each 1 tick
    //        timer1.Increment = 2;    // each 2 ticks
    //
    // ----------------------------------------------------

    this.Increment        = 1;

    // ----------------------------------------------------
    // Property:    TickCount
    // Description:    The number of times .OnTick() has been
    //        called, ranging from 0 to .Interval
    //
    // Usage:    alert("TickCount=="+timer1.TickCount);
    //
    // ----------------------------------------------------

    this.TickCount        = 0;

    // ----------------------------------------------------
    // Property:    TimeOut
    // Description:    if set to >0, the timer will Stop() 
    //        automatically after N calls to .OnTick()
    //
    // Usage:    timer1.TimeOut = 0;    // tick forever
    //        timer1.TimeOut = 5;    // 5 ticks total
    //
    // ----------------------------------------------------

    this.TimeOut        = 100;

    // ----------------------------------------------------
    // Property:    Paused
    // Description:    if true, timer is currently paused.
    //        .Tick() and .OnTick() do nothing.
    //
    // Usage:    if (timer1.Paused)
    //            timer1.Resume();
    //
    // ----------------------------------------------------

    this.Paused        = false;


// ----------------------------------------------------
// Methods
// ----------------------------------------------------

    // ----------------------------------------------------
    // Method:    Start()
    // Description:    Sets the timer running, calling
    //        the .Tick() function on each tick of 
    //        the timer.
    //
    // Usage:    timer1.Interval = 1;
    //        timer1.TimeOut = 0;
    //        timer1.OnTick = MyAnimationFunction;
    //        timer1.Start();
    // ----------------------------------------------------

    this.Start         = d_Timer_Start;

    // ----------------------------------------------------
    // Method:    Stop()
    // Description:    Makes the d_Timer() stop calling the
    //        Tick() function on each tick of the 
    //        timer.
    //
    // Usage:    if (timer1.Active)
    //            timer1.Stop();
    //
    // ----------------------------------------------------

    this.Stop         = d_Timer_Stop;

    // ----------------------------------------------------
    // Method:    Pause()
    // Description:    Lets the browser timer continue to tick
    //        away in the background, the .Tick()
    //        method temporarily stops updating 
    //        .TickCount and does not call the .OnTick() 
    //        event until .Resume() is called.
    //
    // Usage:    if (!timer1.Paused)
    //            timer1.Pause();
    //
    // ----------------------------------------------------

    this.Pause        = d_Timer_Pause;

    // ----------------------------------------------------
    // Method:    Resume()
    // Description:    The .Tick() method will continue
    //        updating the .TickCount property and
    //        will call the .OnTick() event on each
    //        tick.
    //
    // Usage:    if (timer1.Paused)
    //            timer1.Resume();
    //
    // ----------------------------------------------------

    this.Resume        = d_Timer_Resume;

    // ----------------------------------------------------
    // Method:    Reset()
    // Description:    Sets the .TickCount property to 0
    //
    // Usage:    if (timer1.TickCount>100)
    //            timer1.Reset();
    //
    // ----------------------------------------------------

    this.Reset        = d_Timer_Reset;

    // ----------------------------------------------------
    // Method:    Tick()
    // Description:    This method is called on each tick of 
    //        the browser timer. If !Paused, it updates 
    //        the .TickCount property, calls .OnTick(),
    //        then checks iff .TickCount > .TimeOut.
    //        Iff .TickCount > .TimeOut calls .Stop().
    //
    //
    // Usage:    none. This method is called internally
    //
    // ----------------------------------------------------

    this.Tick        = d_Timer_Tick;


// ----------------------------------------------------
// Events
// ----------------------------------------------------

// re-assign to your own code for additional functionality.
// eg. 
//
//    var timer1;
//    function Go()
//        {
//        timer1 = new d_Timer( 'timer1' );
//        timer1.OnStart = MyStart;
//        timer1.OnTick = MyAnimation;
//        timer1.Start();
//        }
//    function MyStart()
//        {
//        alert('timer1 is about to start');
//        }
//    function MyAnimation()
//        {
//        document.status = 'timer1.TickCount=='+timer1.TickCount;
//        }
//

    this.OnStart        = d_Timer_OnStart;
    this.OnStop        = d_Timer_OnStop;
    this.OnTick        = d_Timer_OnTick;            
    this.OnPause        = d_Timer_OnPause;
    this.OnResume        = d_Timer_OnResume;

// ----------------------------------------------------
// Internal Use Only
// ----------------------------------------------------

    this.Timer        = null;

}

// ------------------------------------------------------------
// End of Object Definition
// ------------------------------------------------------------


// ------------------------------------------------------------
// Method Code
// ------------------------------------------------------------

// ------------------------------------------------------------

function d_Timer_Start()
{
if (!this.Active)
    {
    this.OnStart();
    this.Timer = setInterval( this.Name + ".Tick()", this.Interval );
    this.Active = true;
    }
}

// ------------------------------------------------------------

function d_Timer_Stop()
{
if (this.Active)
    {
    clearInterval( this.Timer );
    this.Active = false;
    this.OnStop();
    }
}

// ------------------------------------------------------------

function d_Timer_Pause()
{
if (this.Active)
    {
    clearInterval( this.Timer );
    this.Paused = true;
    this.OnPause();
    }
}

// ------------------------------------------------------------

function d_Timer_Resume()
{
if (this.Paused)
    {
    this.OnResume();
    this.Timer = setInterval( this.Name + ".Tick()", this.Interval );
    this.Paused = false;
    }
}

// ------------------------------------------------------------

function d_Timer_Reset()
{
this.TickCount = 0;
}

// ------------------------------------------------------------

function d_Timer_Tick()
{
status = this.Name+"::Tick(), count=="+this.TickCount;
if (this.Active)
    {
    this.OnTick();
    this.TickCount += this.Increment;
    if (this.TickCount==this.TimeOut) this.Stop();
    }
}

// ------------------------------------------------------------


// ------------------------------------------------------------
// Default Event Handling Code
// ------------------------------------------------------------

function d_Timer_OnStart()
{
if (d_Timer_Debug) status = "Note: "+this.Name+"::OnStart()";
}

// ------------------------------------------------------------

function d_Timer_OnStop()
{
if (d_Timer_Debug) status = "Note: "+this.Name+"::OnStop()";
}

// ------------------------------------------------------------

function d_Timer_OnPause()
{
if (d_Timer_Debug) status = "Note: "+this.Name+"::OnPause()";
}

// ------------------------------------------------------------

function d_Timer_OnResume()
{
if (d_Timer_Debug) status = "Note: "+this.Name+"::OnResume()";
}

// ------------------------------------------------------------

function d_Timer_OnTick()
{
if (d_Timer_Debug) status = "Note: "+this.Name+"::OnTick(), count=="+this.TickCount;
}

// ------------------------------------------------------------
